import insertSVGstr from '../style/icon/insertsnippet.svg';
import carrotSVGstr from '../style/icon/jupyter_snippetarrow.svg';
import { SearchBar } from './SearchBar';
import { showPreview } from './PreviewSnippet';
// import { CodeSnippetWidget } from './CodeSnippetWidget';
import { Clipboard, Dialog, showDialog } from '@jupyterlab/apputils';
import { CodeCell, MarkdownCell } from '@jupyterlab/cells';
import { PathExt } from '@jupyterlab/coreutils';
// import { ServerConnection } from '@jupyterlab/services';
import { DocumentWidget } from '@jupyterlab/docregistry';
import { FileEditor } from '@jupyterlab/fileeditor';
import { NotebookPanel } from '@jupyterlab/notebook';
import { copyIcon } from '@jupyterlab/ui-components';
import { LabIcon } from '@jupyterlab/ui-components';
import { Widget } from '@lumino/widgets';
// import { MouseEvent } from 'react';
import React from 'react';
import { Drag } from '@lumino/dragdrop';
import { CodeCellModel } from '@jupyterlab/cells';
import { MimeData } from '@lumino/coreutils';
/**
 * The class added to snippet cells
 */
// const CODE_SNIPPET_CELL_CLASS = 'jp-CodeSnippet-cell';
/**
 * The CSS class added to code snippet widget.
 */
const CODE_SNIPPETS_HEADER_CLASS = 'codeSnippetsHeader';
const CODE_SNIPPETS_CONTAINER = 'codeSnippetsContainer';
const DISPLAY_NAME_CLASS = 'expandableContainer-name';
const ELYRA_BUTTON_CLASS = 'jp-button';
const BUTTON_CLASS = 'expandableContainer-button';
const TITLE_CLASS = 'expandableContainer-title';
const ACTION_BUTTONS_WRAPPER_CLASS = 'elyra-expandableContainer-action-buttons';
const ACTION_BUTTON_CLASS = 'expandableContainer-actionButton';
/**
 * The threshold in pixels to start a drag event.
 */
const DRAG_THRESHOLD = 5;
/**
 * A class used to indicate a snippet item.
 */
const CODE_SNIPPET_ITEM = 'codeSnippet-item';
/**
 * The mimetype used for Jupyter cell data.
 */
const JUPYTER_CELL_MIME = 'application/vnd.jupyter.cells';
/**
 * Icons used for snippet
 */
const insertIcon = new LabIcon({
    name: 'custom-ui-compnents:insert',
    svgstr: insertSVGstr
});
const previewIcon = new LabIcon({
    name: 'custom-ui-compnents:preview',
    svgstr: carrotSVGstr
});
/**
 * A React Component for code-snippets display list.
 */
export class CodeSnippetDisplay extends React.Component {
    constructor(props) {
        super(props);
        // Handle code snippet insert into an editor
        this.insertCodeSnippet = async (snippet) => {
            var _a, _b, _c;
            const widget = this.props.getCurrentWidget();
            console.log('current widget: ' + widget);
            const snippetStr = snippet.code.join('\n');
            // if the widget is document widget and it's a file?? in the file editor
            if (widget instanceof DocumentWidget &&
                widget.content instanceof FileEditor) {
                const documentWidget = widget;
                // code editor
                const fileEditor = documentWidget.content.editor;
                const markdownRegex = /^\.(md|mkdn?|mdown|markdown)$/;
                if (PathExt.extname(widget.context.path).match(markdownRegex) !== null) {
                    // Wrap snippet into a code block when inserting it into a markdown file
                    fileEditor.replaceSelection('```' + snippet.language + '\n' + snippetStr + '\n```');
                }
                else if (widget.constructor.name === 'PythonFileEditor') {
                    this.verifyLanguageAndInsert(snippet, 'python', fileEditor);
                }
                else {
                    fileEditor.replaceSelection(snippetStr);
                }
            }
            else if (widget instanceof NotebookPanel) {
                const notebookWidget = widget;
                const notebookCell = notebookWidget.content.activeCell;
                // editor
                const notebookCellEditor = notebookCell.editor;
                if (notebookCell instanceof CodeCell) {
                    const kernelInfo = await ((_b = (_a = notebookWidget.sessionContext.session) === null || _a === void 0 ? void 0 : _a.kernel) === null || _b === void 0 ? void 0 : _b.info);
                    const kernelLanguage = ((_c = kernelInfo) === null || _c === void 0 ? void 0 : _c.language_info.name) || '';
                    this.verifyLanguageAndInsert(snippet, kernelLanguage, notebookCellEditor);
                }
                else if (notebookCell instanceof MarkdownCell) {
                    // Wrap snippet into a code block when inserting it into a markdown cell
                    notebookCellEditor.replaceSelection('```' + snippet.language + '\n' + snippetStr + '\n```');
                }
                else {
                    notebookCellEditor.replaceSelection(snippetStr);
                }
            }
            else {
                this.showErrDialog('Code snippet insert failed: Unsupported widget');
            }
        };
        // // Handle deleting code snippet
        // private deleteCodeSnippet = async (snippet: ICodeSnippet): Promise<void> => {
        //   const name = snippet.name;
        //   // const url = 'elyra/metadata/code-snippets/' + name;
        //   this.props.openCodeSnippetEditor({ namespace: name, codeSnippet: snippet });
        // };
        // Handle language compatibility between code snippet and editor
        this.verifyLanguageAndInsert = async (snippet, editorLanguage, editor) => {
            const snippetStr = snippet.code.join('\n');
            if (editorLanguage &&
                snippet.language.toLowerCase() !== editorLanguage.toLowerCase()) {
                const result = await this.showWarnDialog(editorLanguage, snippet.name);
                if (result.button.accept) {
                    editor.replaceSelection(snippetStr);
                }
            }
            else {
                // Language match or editorLanguage is unavailable
                editor.replaceSelection(snippetStr);
            }
        };
        // Display warning dialog when inserting a code snippet incompatible with editor's language
        this.showWarnDialog = async (editorLanguage, snippetName) => {
            return showDialog({
                title: 'Warning',
                body: 'Code snippet "' +
                    snippetName +
                    '" is incompatible with ' +
                    editorLanguage +
                    '. Continue?',
                buttons: [Dialog.cancelButton(), Dialog.okButton()]
            });
        };
        // Display error dialog when inserting a code snippet into unsupported widget (i.e. not an editor)
        this.showErrDialog = (errMsg) => {
            return showDialog({
                title: 'Error',
                body: errMsg,
                buttons: [Dialog.okButton()]
            });
        };
        // Pick color for side of snippet box based on number of code lines
        // private codeLines = (codeSnippet: ICodeSnippet): string => {
        //   let i;
        //   let counter = 0;
        //   for (i = 0; i < codeSnippet.code[0].length; i++) {
        //     if (codeSnippet.code[0][i] === '\n') {
        //       counter++;
        //     }
        //   }
        //   counter += 1;
        //   console.log(counter);
        //   return 'LOC\t\t' + counter;
        // };
        // Insert 6 dots on hover
        this.dragHoverStyle = (id) => {
            const _id = parseInt(id, 10);
            document
                .getElementsByClassName('drag-hover')[_id].classList.add('drag-hover-selected');
        };
        // Remove 6 dots off hover
        this.dragHoverStyleRemove = (id) => {
            const _id = parseInt(id, 10);
            document
                .getElementsByClassName('drag-hover')[_id].classList.remove('drag-hover-selected');
        };
        // Grey out snippet and include blue six dots when snippet is previewing (clicked)
        this.snippetClicked = (id) => {
            const _id = parseInt(id, 10);
            if (document
                .getElementsByClassName('drag-hover')[_id].classList.contains('drag-hover-clicked')) {
                document
                    .getElementsByClassName('drag-hover')[_id].classList.remove('drag-hover-clicked');
            }
            else {
                document
                    .getElementsByClassName('drag-hover')[_id].classList.add('drag-hover-clicked');
            }
            if (document
                .getElementsByClassName(CODE_SNIPPET_ITEM)[_id].classList.contains('codeSnippet-item-clicked')) {
                document
                    .getElementsByClassName(CODE_SNIPPET_ITEM)[_id].classList.remove('codeSnippet-item-clicked');
            }
            else {
                document
                    .getElementsByClassName(CODE_SNIPPET_ITEM)[_id].classList.add('codeSnippet-item-clicked');
            }
        };
        // Bold text in snippet name based on search
        this.boldNameOnSearch = (filter, displayed) => {
            const name = displayed;
            if (filter !== '') {
                const startIndex = name.indexOf(filter);
                const endIndex = startIndex + filter.length;
                const start = name.substring(0, startIndex);
                const bolded = name.substring(startIndex, endIndex);
                const end = name.substring(endIndex);
                return (React.createElement("span", null,
                    start,
                    React.createElement("mark", { className: "jp-search-bolding" }, bolded),
                    end));
            }
            return name;
        };
        // Render display of code snippet list
        // To get the variety of color based on code length just append -long to CODE_SNIPPET_ITEM
        this.renderCodeSnippet = (codeSnippet, id) => {
            const buttonClasses = [ELYRA_BUTTON_CLASS, BUTTON_CLASS].join(' ');
            const displayName = '[' + codeSnippet.language + '] ' + codeSnippet.name;
            const actionButtons = [
                {
                    title: 'Copy',
                    icon: copyIcon,
                    onClick: () => {
                        Clipboard.copyToSystem(codeSnippet.code.join('\n'));
                    }
                },
                {
                    title: 'Insert',
                    icon: insertIcon,
                    onClick: () => {
                        this.insertCodeSnippet(codeSnippet);
                    }
                },
                {
                    title: 'Preview',
                    icon: previewIcon,
                    onClick: () => {
                        showPreview({
                            id: parseInt(id, 10),
                            title: displayName,
                            body: new PreviewHandler(codeSnippet),
                            codeSnippet: codeSnippet
                        }, this.props.openCodeSnippetEditor);
                        this.snippetClicked(id);
                    }
                }
            ];
            /** TODO: if the type is a cell then display cell */
            // type of code snippet: plain code or cell
            return (React.createElement("div", { key: codeSnippet.name, className: CODE_SNIPPET_ITEM, id: id, onMouseOver: () => {
                    this.dragHoverStyle(id);
                }, onMouseOut: () => {
                    this.dragHoverStyleRemove(id);
                } },
                React.createElement("div", { className: "drag-hover", title: "Drag to move", id: id, onMouseDown: (event) => {
                        this.handleDragSnippet(event);
                    } }),
                React.createElement("div", null,
                    React.createElement("div", { key: displayName, className: TITLE_CLASS },
                        React.createElement("span", { id: id, title: codeSnippet.name, className: DISPLAY_NAME_CLASS, onClick: () => {
                                showPreview({
                                    id: parseInt(id, 10),
                                    title: displayName,
                                    body: new PreviewHandler(codeSnippet),
                                    codeSnippet: codeSnippet
                                }, this.props.openCodeSnippetEditor);
                                this.snippetClicked(id);
                            } }, this.boldNameOnSearch(this.state.filterValue, displayName)),
                        React.createElement("div", { className: ACTION_BUTTONS_WRAPPER_CLASS }, actionButtons.map((btn) => {
                            return (React.createElement("button", { key: btn.title, title: btn.title, className: buttonClasses + ' ' + ACTION_BUTTON_CLASS, onClick: () => {
                                    if (btn.title === 'Copy') {
                                        alert('saved to clipboard');
                                    }
                                    btn.onClick();
                                } },
                                React.createElement(btn.icon.react, { tag: "span", elementPosition: "center", width: "16px" })));
                        }))))));
        };
        this.filterSnippets = (filterValue) => {
            const newSnippets = this.props.codeSnippets.filter(codeSnippet => codeSnippet.name.includes(filterValue) ||
                codeSnippet.language.includes(filterValue));
            this.setState({ codeSnippets: newSnippets, filterValue: filterValue }, () => {
                console.log('CodeSnippets are successfully filtered.');
            });
        };
        this.state = { codeSnippets: this.props.codeSnippets, filterValue: '' };
        this._drag = null;
        this._dragData = null;
        this.handleDragMove = this.handleDragMove.bind(this);
        this._evtMouseUp = this._evtMouseUp.bind(this);
    }
    handleDragSnippet(event) {
        const { button } = event;
        // if button is not the left click
        if (!(button === 0)) {
            return;
        }
        const target = event.target;
        this._dragData = {
            pressX: event.clientX,
            pressY: event.clientY,
            dragImage: target.parentNode.cloneNode(true)
        };
        // add CSS style
        this._dragData.dragImage.classList.add('jp-codesnippet-drag-image');
        target.addEventListener('mouseup', this._evtMouseUp, true);
        target.addEventListener('mousemove', this.handleDragMove, true);
        event.preventDefault();
        // event.stopPropagation();
    }
    _evtMouseUp(event) {
        event.preventDefault();
        event.stopPropagation();
        const target = event.target;
        target.removeEventListener('mousemove', this.handleDragMove, true);
        target.removeEventListener('mouseup', this._evtMouseUp, true);
    }
    handleDragMove(event) {
        // event.preventDefault();
        // event.stopPropagation();
        const data = this._dragData;
        if (data &&
            this.shouldStartDrag(data.pressX, data.pressY, event.clientX, event.clientY)) {
            const idx = event.target.id;
            const codeSnippet = this.state.codeSnippets[parseInt(idx)];
            this.startDrag(data.dragImage, codeSnippet, event.clientX, event.clientY);
        }
    }
    /**
     * Detect if a drag event should be started. This is down if the
     * mouse is moved beyond a certain distance (DRAG_THRESHOLD).
     *
     * @param prevX - X Coordinate of the mouse pointer during the mousedown event
     * @param prevY - Y Coordinate of the mouse pointer during the mousedown event
     * @param nextX - Current X Coordinate of the mouse pointer
     * @param nextY - Current Y Coordinate of the mouse pointer
     */
    shouldStartDrag(prevX, prevY, nextX, nextY) {
        const dx = Math.abs(nextX - prevX);
        const dy = Math.abs(nextY - prevY);
        return dx >= DRAG_THRESHOLD || dy >= DRAG_THRESHOLD;
    }
    async startDrag(dragImage, codeSnippet, clientX, clientY) {
        const target = event.target;
        const modelFactory = new ModelFactory();
        const model = modelFactory.createCodeCell({});
        model.value.text = codeSnippet.code.join('\n');
        model.metadata;
        const selected = [model.toJSON()];
        this._drag = new Drag({
            mimeData: new MimeData(),
            dragImage: dragImage,
            supportedActions: 'copy-move',
            proposedAction: 'copy',
            source: this
        });
        this._drag.mimeData.setData(JUPYTER_CELL_MIME, selected);
        const textContent = codeSnippet.code.join('\n');
        this._drag.mimeData.setData('text/plain', textContent);
        // Remove mousemove and mouseup listeners and start the drag.
        target.removeEventListener('mousemove', this.handleDragMove, true);
        target.removeEventListener('mouseup', this._evtMouseUp, true);
        return this._drag.start(clientX, clientY).then(() => {
            console.log('drag done');
            this.dragHoverStyleRemove(codeSnippet.id.toString());
            this._drag = null;
            this._dragData = null;
        });
    }
    static getDerivedStateFromProps(props, state) {
        console.log('udpating display!');
        console.log(props);
        console.log(state);
        if (props.codeSnippets !== state.codeSnippets && state.filterValue === '') {
            return {
                codeSnippets: props.codeSnippets,
                filterValue: ''
            };
        }
        return null;
    }
    render() {
        return (React.createElement("div", null,
            React.createElement(SearchBar, { onFilter: this.filterSnippets }),
            React.createElement("header", { className: CODE_SNIPPETS_HEADER_CLASS }, 'Snippets'),
            React.createElement("div", { className: CODE_SNIPPETS_CONTAINER },
                React.createElement("div", null, this.state.codeSnippets.map((codeSnippet, id) => this.renderCodeSnippet(codeSnippet, id.toString()))))));
    }
}
class PreviewHandler extends Widget {
    constructor(codeSnippet) {
        super({ node: Private.createPreviewNode(codeSnippet) });
    }
}
class Private {
    static createPreviewContent(codeSnippet) {
        const body = document.createElement('div');
        const previewContainer = document.createElement('div');
        const descriptionContainer = document.createElement('div');
        const descriptionTitle = document.createElement('h4');
        const description = document.createElement('text');
        const preview = document.createElement('text');
        previewContainer.className = 'jp-preview-text';
        descriptionContainer.className = 'jp-preview-description-container';
        descriptionTitle.className = 'jp-preview-description-title';
        description.className = 'jp-preview-description';
        preview.className = 'jp-preview-textarea';
        descriptionTitle.textContent = 'DESCRIPTION';
        description.textContent = codeSnippet.description;
        preview.textContent = codeSnippet.code.join('\n');
        descriptionContainer.appendChild(descriptionTitle);
        descriptionContainer.appendChild(description);
        previewContainer.appendChild(descriptionContainer);
        previewContainer.appendChild(preview);
        body.append(previewContainer);
        return body;
    }
    /**
     * Create structure for preview of snippet data.
     */
    static createPreviewNode(codeSnippet) {
        return this.createPreviewContent(codeSnippet);
    }
}
/**
 * The default implementation of an `IModelFactory`.
 */
export class ModelFactory {
    /**
     * Create a new code cell.
     *
     * @param source - The data to use for the original source data.
     *
     * @returns A new code cell. If a source cell is provided, the
     *   new cell will be initialized with the data from the source.
     *   If the contentFactory is not provided, the instance
     *   `codeCellContentFactory` will be used.
     */
    createCodeCell(options) {
        if (!options.contentFactory) {
            options.contentFactory = this.codeCellContentFactory;
        }
        return new CodeCellModel(options);
    }
}
