import { CodeEditor } from '@jupyterlab/codeeditor';
import { ReactWidget, showDialog, Dialog } from '@jupyterlab/apputils';
import React from 'react';
import { Button } from '@jupyterlab/ui-components';
/**
 * CSS style classes
 */
const CODE_SNIPPET_EDITOR = 'jp-codeSnippet-editor';
const CODE_SNIPPET_EDITOR_NAME_LABEL = 'jp-snippet-editor-name-label';
const CODE_SNIPPET_EDITOR_LABEL_ACTIVE = 'jp-snippet-editor-label-active';
const CODE_SNIPPET_EDITOR_INPUT_ACTIVE = 'jp-snippet-editor-active';
const CODE_SNIPPET_EDITOR_NAME_INPUT = 'jp-snippet-editor-name';
const CODE_SNIPPET_EDITOR_DESC_LABEL = 'jp-snippet-editor-description-label';
const CODE_SNIPPET_EDITOR_DESC_INPUT = 'jp-snippet-editor-description';
const CODE_SNIPPET_EDITOR_LANG_INPUT = 'jp-snippet-editor-language';
const CODE_SNIPPET_EDITOR_MIRROR = 'jp-codeSnippetInput-editor';
const EDITOR_DIRTY_CLASS = 'jp-mod-dirty';
export class CodeSnippetEditor extends ReactWidget {
    constructor(contentsService, editorServices, tracker, codeSnippetWidget, args) {
        super();
        this.addClass(CODE_SNIPPET_EDITOR);
        this.contentsService = contentsService;
        this.editorServices = editorServices;
        this.tracker = tracker;
        this._codeSnippet = args;
        this.oldCodeSnippetName = args.name;
        this.saved = true;
        this._hasRefreshedSinceAttach = false;
        this.codeSnippetWidget = codeSnippetWidget;
        this.renderCodeInput = this.renderCodeInput.bind(this);
        this.handleInputFieldChange = this.handleInputFieldChange.bind(this);
        this.activateCodeMirror = this.activateCodeMirror.bind(this);
        this.saveChange = this.saveChange.bind(this);
        this.updateSnippet = this.updateSnippet.bind(this);
    }
    get codeSnippet() {
        return this._codeSnippet;
    }
    deactivateEditor(event) {
        let target = event.target;
        while (target && target.parentElement) {
            if (target.classList.contains(CODE_SNIPPET_EDITOR_MIRROR) ||
                target.classList.contains(CODE_SNIPPET_EDITOR_NAME_INPUT) ||
                target.classList.contains(CODE_SNIPPET_EDITOR_DESC_INPUT)) {
                break;
            }
            target = target.parentElement;
        }
        const nameLabel = document.querySelector(`.${CODE_SNIPPET_EDITOR}-${this._codeSnippet.id} .${CODE_SNIPPET_EDITOR_NAME_LABEL}`);
        const nameInput = document.querySelector(`.${CODE_SNIPPET_EDITOR}-${this._codeSnippet.id} .${CODE_SNIPPET_EDITOR_NAME_INPUT}`);
        const descriptionLabel = document.querySelector(`.${CODE_SNIPPET_EDITOR}-${this._codeSnippet.id} .${CODE_SNIPPET_EDITOR_DESC_LABEL}`);
        const descriptionInput = document.querySelector(`.${CODE_SNIPPET_EDITOR}-${this._codeSnippet.id} .${CODE_SNIPPET_EDITOR_DESC_INPUT}`);
        const editor = document.querySelector(`.${CODE_SNIPPET_EDITOR}-${this._codeSnippet.id} #code-${this._codeSnippet.id}`);
        if (target.classList.contains(CODE_SNIPPET_EDITOR_NAME_INPUT)) {
            this.deactivateDescriptionField(descriptionLabel, descriptionInput);
            this.deactivateCodeMirror(editor);
        }
        else if (target.classList.contains(CODE_SNIPPET_EDITOR_DESC_INPUT)) {
            this.deactivateNameField(nameLabel, nameInput);
            this.deactivateCodeMirror(editor);
        }
        else if (target.classList.contains(CODE_SNIPPET_EDITOR_MIRROR)) {
            this.deactivateNameField(nameLabel, nameInput);
            this.deactivateDescriptionField(descriptionLabel, descriptionInput);
        }
        else {
            this.deactivateNameField(nameLabel, nameInput);
            this.deactivateDescriptionField(descriptionLabel, descriptionInput);
            this.deactivateCodeMirror(editor);
        }
    }
    deactivateNameField(nameLabel, nameInput) {
        if (nameLabel.classList.contains(CODE_SNIPPET_EDITOR_LABEL_ACTIVE) &&
            nameInput.classList.contains(CODE_SNIPPET_EDITOR_INPUT_ACTIVE)) {
            nameLabel.classList.remove(CODE_SNIPPET_EDITOR_LABEL_ACTIVE);
            nameInput.classList.remove(CODE_SNIPPET_EDITOR_INPUT_ACTIVE);
        }
    }
    deactivateDescriptionField(descriptionLabel, descriptionInput) {
        if (descriptionLabel.classList.contains(CODE_SNIPPET_EDITOR_LABEL_ACTIVE) &&
            descriptionInput.classList.contains(CODE_SNIPPET_EDITOR_INPUT_ACTIVE)) {
            descriptionLabel.classList.remove(CODE_SNIPPET_EDITOR_LABEL_ACTIVE);
            descriptionInput.classList.remove(CODE_SNIPPET_EDITOR_INPUT_ACTIVE);
        }
    }
    activeFieldState(event) {
        const target = event.target;
        let label;
        // if target is a description input, activate description label; if target is a name input, activate name label
        if (target.classList.contains(CODE_SNIPPET_EDITOR_DESC_INPUT)) {
            label = document.querySelector(`.${CODE_SNIPPET_EDITOR}-${this._codeSnippet.id} .${CODE_SNIPPET_EDITOR_DESC_LABEL}`);
        }
        else if (target.classList.contains(CODE_SNIPPET_EDITOR_NAME_INPUT)) {
            label = document.querySelector(`.${CODE_SNIPPET_EDITOR}-${this._codeSnippet.id} .${CODE_SNIPPET_EDITOR_NAME_LABEL}`);
        }
        if (!target.classList.contains(CODE_SNIPPET_EDITOR_INPUT_ACTIVE)) {
            target.classList.add(CODE_SNIPPET_EDITOR_INPUT_ACTIVE);
            label.classList.add(CODE_SNIPPET_EDITOR_LABEL_ACTIVE);
        }
    }
    onUpdateRequest(msg) {
        super.onUpdateRequest(msg);
        if (!this.editor &&
            document.getElementById('code-' + this._codeSnippet.id)) {
            const editorFactory = this.editorServices.factoryService.newInlineEditor;
            const getMimeTypeByLanguage = this.editorServices.mimeTypeService
                .getMimeTypeByLanguage;
            this.editor = editorFactory({
                host: document.getElementById('code-' + this._codeSnippet.id),
                model: new CodeEditor.Model({
                    value: this._codeSnippet.code.join('\n'),
                    mimeType: getMimeTypeByLanguage({
                        name: this._codeSnippet.language,
                        codemirror_mode: this._codeSnippet.language
                    })
                })
            });
            this.editor.model.value.changed.connect((args) => {
                this._codeSnippet.code = args.text.split('\n');
                if (!this.title.className.includes(EDITOR_DIRTY_CLASS)) {
                    this.title.className += ` ${EDITOR_DIRTY_CLASS}`;
                }
                this.saved = false;
            });
        }
        if (this.isVisible) {
            this._hasRefreshedSinceAttach = true;
            this.editor.refresh();
        }
    }
    onAfterAttach(msg) {
        super.onAfterAttach(msg);
        this._hasRefreshedSinceAttach = false;
        if (this.isVisible) {
            this.update();
        }
        window.addEventListener('beforeunload', e => {
            if (!this.saved) {
                e.preventDefault();
                e.returnValue = '';
            }
        });
    }
    onAfterShow(msg) {
        if (!this._hasRefreshedSinceAttach) {
            this.update();
        }
    }
    /**
     * Initial focus on the editor when it gets activated!
     * @param msg
     */
    onActivateRequest(msg) {
        this.editor.focus();
    }
    onCloseRequest(msg) {
        if (!this.saved) {
            showDialog({
                title: 'Close without saving?',
                body: (React.createElement("p", null,
                    ' ',
                    `"${this._codeSnippet.name}" has unsaved changes, close without saving?`,
                    ' ')),
                buttons: [Dialog.cancelButton(), Dialog.okButton()]
            }).then((response) => {
                if (response.button.accept) {
                    this.dispose();
                    super.onCloseRequest(msg);
                }
            });
        }
        else {
            this.dispose();
            super.onCloseRequest(msg);
        }
    }
    /**
     * Visualize the editor more look like an editor
     * @param event
     */
    activateCodeMirror(event) {
        let target = event.target;
        while (target && target.parentElement) {
            if (target.classList.contains(CODE_SNIPPET_EDITOR_MIRROR)) {
                break;
            }
            target = target.parentElement;
        }
        const editor = document.querySelector(`.${CODE_SNIPPET_EDITOR}-${this._codeSnippet.id} #code-${this._codeSnippet.id}`);
        if (target.classList.contains(CODE_SNIPPET_EDITOR_MIRROR)) {
            if (!editor.classList.contains('active')) {
                editor.classList.add('active');
            }
        }
    }
    deactivateCodeMirror(editor) {
        if (editor.classList.contains('active')) {
            editor.classList.remove('active');
        }
    }
    handleInputFieldChange(event) {
        if (!this.title.className.includes(EDITOR_DIRTY_CLASS)) {
            this.title.className += ` ${EDITOR_DIRTY_CLASS}`;
        }
        const target = event.target;
        if (!target.classList.contains('FieldChanged')) {
            target.classList.add('FieldChanged');
        }
        this.saved = false;
    }
    saveChange(event) {
        this.updateSnippet();
    }
    async updateSnippet() {
        const name = document.querySelector(`.${CODE_SNIPPET_EDITOR}-${this._codeSnippet.id} .${CODE_SNIPPET_EDITOR_NAME_INPUT}`).value;
        const description = document.querySelector(`.${CODE_SNIPPET_EDITOR}-${this._codeSnippet.id} .${CODE_SNIPPET_EDITOR_DESC_INPUT}`).value;
        const language = document.querySelector(`.${CODE_SNIPPET_EDITOR}-${this._codeSnippet.id} .${CODE_SNIPPET_EDITOR_LANG_INPUT}`).value;
        this._codeSnippet.name = name;
        this._codeSnippet.description = description;
        this._codeSnippet.language = language;
        this.saved = true;
        const oldPath = 'snippets/' + this.oldCodeSnippetName + '.json';
        const newPath = 'snippets/' + this._codeSnippet.name + '.json';
        if (newPath !== oldPath) {
            // renaming code snippet
            try {
                await this.contentsService.rename(oldPath, newPath);
            }
            catch (error) {
                console.log('duplicate name!');
                await showDialog({
                    title: 'Duplicate Name of Code Snippet',
                    body: React.createElement("p", null,
                        " ",
                        `"${newPath}" already exists.`,
                        " "),
                    buttons: [Dialog.cancelButton()]
                });
                return;
            }
            // set new name as an old name
            this.oldCodeSnippetName = this._codeSnippet.name;
        }
        await this.contentsService.save(newPath, {
            type: 'file',
            format: 'text',
            content: JSON.stringify(this._codeSnippet)
        });
        // remove the dirty state
        this.title.className = this.title.className.replace(` ${EDITOR_DIRTY_CLASS}`, '');
        // change label
        this.title.label =
            '[' + this._codeSnippet.language + '] ' + this._codeSnippet.name;
        // update tracker
        this.tracker.save(this);
        // update the display in code snippet explorer
        this.codeSnippetWidget.updateCodeSnippets();
    }
    /**
     * TODO: clean CSS style class - "Use constant"
     */
    renderCodeInput() {
        return (React.createElement("section", { className: "jp-codeSnippetInputArea-editor", onMouseDown: this.activateCodeMirror },
            React.createElement("div", { className: CODE_SNIPPET_EDITOR_MIRROR, id: 'code-' + this._codeSnippet.id.toString() })));
    }
    render() {
        return (React.createElement("div", { className: "jp-codeSnippetInputArea", onMouseDown: (event) => {
                // this.deactivateEditor(event);
                this.deactivateEditor(event);
            } },
            React.createElement("span", { className: "jp-snippet-editor-title" }, "Edit Code Snippet"),
            React.createElement("section", { className: "jp-snippet-editor-metadata" },
                React.createElement("label", { className: "jp-snippet-editor-name-label" }, "Name"),
                React.createElement("input", { className: "jp-snippet-editor-name", defaultValue: this._codeSnippet.name, type: "text", onMouseDown: (event) => this.activeFieldState(event), onChange: (event) => {
                        this.handleInputFieldChange(event);
                    } }),
                React.createElement("label", { className: "jp-snippet-editor-description-label" }, "Description"),
                React.createElement("input", { className: "jp-snippet-editor-description", defaultValue: this._codeSnippet.description, type: "text", onMouseDown: (event) => this.activeFieldState(event), onChange: (event) => {
                        this.handleInputFieldChange(event);
                    } }),
                React.createElement("select", { className: "jp-snippet-editor-language", defaultValue: this._codeSnippet.language, name: "languages" },
                    React.createElement("option", { className: "jp-snippet-editor-options", value: "python" }, "python"),
                    React.createElement("option", { className: "jp-snippet-editor-options", value: "R" }, "R"),
                    React.createElement("option", { className: "jp-snippet-editor-options", value: "Scala" }, "Scala"),
                    React.createElement("option", { className: "jp-snippet-editor-options", value: "Other" }, "Other"))),
            React.createElement("span", { className: "jp-codeSnippetInputArea-editorTitle" }, "Code"),
            this.renderCodeInput(),
            React.createElement(Button, { className: "saveBtn", onClick: this.saveChange }, "Save")));
    }
}
