import '../style/index.css';
import { ReactWidget, WidgetTracker } from '@jupyterlab/apputils';
import { IDocumentManager } from '@jupyterlab/docmanager';
import { IRenderMimeRegistry } from '@jupyterlab/rendermime';
import { Widget, PanelLayout } from '@lumino/widgets';
import { Message } from '@lumino/messaging';
import { Signal } from '@lumino/signaling';
import React from 'react';
import { CodeSnippetContentsService, ICodeSnippet } from './CodeSnippetContentsService';
import { CodeSnippetWidgetModel } from './CodeSnippetWidgetModel';
import { JupyterFrontEnd } from '@jupyterlab/application';
import { IEditorServices } from '@jupyterlab/codeeditor';
/**
 * A widget for Code Snippets.
 */
export declare class CodeSnippetWidget extends ReactWidget {
    getCurrentWidget: () => Widget;
    private _codeSnippetWidgetModel;
    _codeSnippets: ICodeSnippet[];
    renderCodeSnippetsSignal: Signal<this, ICodeSnippet[]>;
    app: JupyterFrontEnd;
    codeSnippetManager: CodeSnippetContentsService;
    constructor(getCurrentWidget: () => Widget, app: JupyterFrontEnd, editorServices: IEditorServices);
    fetchData(): Promise<ICodeSnippet[]>;
    updateCodeSnippets(): void;
    onAfterShow(msg: Message): void;
    get codeSnippetWidgetModel(): CodeSnippetWidgetModel;
    set codeSnippets(codeSnippets: ICodeSnippet[]);
    openCodeSnippetEditor(args: any): void;
    /**
     * Handle the DOM events for the widget.
     *
     * @param event - The DOM event sent to the widget.
     *
     * #### Notes
     * This method implements the DOM `EventListener` interface and is
     * called in response to events on the notebook panel's node. It should
     * not be called directly by user code.
     */
    handleEvent(event: Event): void;
    /**
     * A message handler invoked on an `'after-attach'` message.
     * @param msg
     */
    protected onAfterAttach(msg: Message): void;
    /**
     * Handle `before-detach` messages for the widget.
     * @param msg
     */
    protected onBeforeDetach(msg: Message): void;
    /**
     * Find the snippet index containing the target html element.
     *
     * #### Notes
     * Returns -1 if the cell is not found.
     */
    private _findSnippet;
    private _evtMouseDown;
    /**
     * Handle the `'lm-dragenter'` event for the widget.
     */
    private _evtDragEnter;
    /**
     * Handle the `'lm-dragleave'` event for the widget.
     */
    private _evtDragLeave;
    /**
     * Handle the `'lm-dragover'` event for the widget.
     */
    private _evtDragOver;
    /**
     * Hanlde the `'lm-drop'` event for the widget.
     */
    private _evtDrop;
    moveCodeSnippet(srcIdx: number, targetIdx: number): void;
    render(): React.ReactElement;
}
/**
 * A custom panel layout for the notebook.
 */
export declare class SnippetPanelLayout extends PanelLayout {
    /**
     * A message handler invoked on an `'update-request'` message.
     *
     * #### Notes
     * This is a reimplementation of the base class method,
     * and is a no-op.
     */
    protected onUpdateRequest(msg: Message): void;
}
/**
 * A namespace for CodeSnippet statics.
 */
export declare namespace CodeSnippetWidget {
    /**
     * Interface describing table of contents widget options.
     */
    interface IOptions {
        /**
         * Application document manager.
         */
        docmanager: IDocumentManager;
        /**
         * Application rendered MIME type.
         */
        rendermime: IRenderMimeRegistry;
    }
    /**
     * The dialog widget tracker.
     */
    const tracker: WidgetTracker<CodeSnippetWidget>;
}
