import '../style/index.css';
import { Widget, PanelLayout, Panel } from '@lumino/widgets';
import { WidgetTracker, ReactWidget } from '@jupyterlab/apputils';
import { MessageLoop } from '@lumino/messaging';
import { PromiseDelegate } from '@lumino/coreutils';
import { ArrayExt } from '@lumino/algorithm';
/**
 * The class name for confirmation box
 */
const CONFIRM_CLASS = 'jp-undo-delete';
/**
 * Create and show a dialog.
 *
 * @param options - The dialog setup options.
 *
 * @returns A promise that resolves with whether the dialog was accepted.
 */
export function showUndoMessage(options = {}) {
    console.log(options);
    const confirmMessage = new UndoDelete(options);
    return confirmMessage.launch();
}
/**
 * A widget used to show undo delete message.
 */
export class UndoDelete extends Widget {
    constructor(options = {}) {
        super();
        this.addClass(CONFIRM_CLASS);
        const renderer = UndoDelete.defaultRenderer;
        this._host = options.host || document.body;
        const layout = (this.layout = new PanelLayout());
        const content = new Panel();
        content.addClass('jp-undo-delete-content');
        layout.addWidget(content);
        const body = renderer.createBody(options.body || '');
        // body.addClass('jp-Message-body');
        // const icon = renderer.createIcon();
        // content.addWidget(icon);
        content.addWidget(body);
        if (UndoDelete.tracker.size > 0) {
            console.log('hihihihi');
            const previous = UndoDelete.tracker.currentWidget;
            previous.reject();
            UndoDelete.tracker.dispose();
        }
        void UndoDelete.tracker.add(this);
    }
    /**
     * Launch the dialog as a modal window.
     *
     * @returns a promise that resolves with the result of the dialog.
     */
    launch() {
        // Return the existing dialog if already open.
        if (this._promise) {
            return this._promise.promise;
        }
        const promise = (this._promise = new PromiseDelegate());
        const promises = Promise.all(Private.launchQueue);
        Private.launchQueue.push(this._promise.promise);
        console.log(Private.launchQueue);
        return promises.then(() => {
            console.log('here44');
            Widget.attach(this, this._host);
            return promise.promise;
        });
    }
    /**
     * Resolve the current dialog.
     *
     * @param index - An optional index to the button to resolve.
     *
     * #### Notes
     * Will default to the defaultIndex.
     * Will resolve the current `show()` with the button value.
     * Will be a no-op if the dialog is not shown.
     */
    resolve() {
        if (!this._promise) {
            return;
        }
        this._resolve();
    }
    /**
     * Reject the current dialog with a default reject value.
     *
     * #### Notes
     * Will be a no-op if the dialog is not shown.
     */
    reject() {
        if (!this._promise) {
            return;
        }
        this._resolve();
    }
    /**
     * Resolve a button item.
     */
    _resolve() {
        // Prevent loopback.
        const promise = this._promise;
        if (!promise) {
            this.dispose();
            return;
        }
        this._promise = null;
        ArrayExt.removeFirstOf(Private.launchQueue, promise.promise);
        this.dispose();
        promise.resolve();
    }
    /**
     * Dispose of the resources used by the dialog.
     */
    dispose() {
        const promise = this._promise;
        if (promise) {
            this._promise = null;
            promise.reject(void 0);
            ArrayExt.removeFirstOf(Private.launchQueue, promise.promise);
        }
        super.dispose();
    }
    /**
     * A message handler invoked on a `'close-request'` message.
     */
    onCloseRequest(msg) {
        if (this._promise) {
            this.reject();
        }
        super.onCloseRequest(msg);
    }
}
(function (UndoDelete) {
    class Renderer {
        /**
         * Create the body of the dialog.
         *
         * @param value - The input value for the body.
         *
         * @returns A widget for the body.
         */
        createBody(value) {
            let body;
            if (typeof value === 'string') {
                body = new Widget({ node: document.createElement('span') });
                body.node.textContent = value;
            }
            else if (value instanceof Widget) {
                body = value;
            }
            else {
                body = ReactWidget.create(value);
                // Immediately update the body even though it has not yet attached in
                // order to trigger a render of the DOM nodes from the React element.
                MessageLoop.sendMessage(body, Widget.Msg.UpdateRequest);
            }
            // const iconNode = new Widget({ node: document.createElement('div') });
            // iconNode.title.icon = checkIcon;
            // body.
            body.addClass('jp-undo-delete-body');
            // Styling.styleNode(body.node);
            return body;
        }
    }
    UndoDelete.Renderer = Renderer;
    /**
     * The default renderer instance.
     */
    UndoDelete.defaultRenderer = new Renderer();
    /**
     * The dialog widget tracker.
     */
    UndoDelete.tracker = new WidgetTracker({
        namespace: '@jupyterlab/code_snippet:UndoDeleteWidget'
    });
})(UndoDelete || (UndoDelete = {}));
/**
 * The namespace for module private data.
 */
var Private;
(function (Private) {
    /**
     * The queue for launching dialogs.
     */
    Private.launchQueue = [];
})(Private || (Private = {}));
